@@ -665,6 +665,9 @@ CldsPlayer::rewind (int subsong)
 void
 CldsPlayer::playsound (int inst_number, int channel_number, int tunehigh)
 {
+  if ((unsigned int)inst_number > numpatch-1) {
+      return;
+  }
   Channel *c = &channel[channel_number];    // current channel
   SoundBank *i = &soundbank[inst_number];   // current instrument
   unsigned int regnum = op_table[channel_number];   // channel's OPL2 register
