@@ -38,14 +38,18 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <audacious/plugin.h>
-#include <audacious/auddrct.h>
+
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <pthread.h>
 #include <assert.h>
 #include <math.h>
 
+#include <audacious/configdb.h>
+#include <audacious/debug.h>
+#include <audacious/drct.h>
+#include <audacious/plugin.h>
+
 #include "alarm.h"
 #include "interface.h"
 #include "callbacks.h"
@@ -181,11 +185,11 @@ void alarm_save(GtkButton *w, gpointer data)
    /*
     * update the live values and write them out
     */
-   alarm_h = alarm_conf.default_hour = 
+   alarm_h = alarm_conf.default_hour =
      gtk_spin_button_get_value_as_int(alarm_conf.alarm_h);
    aud_cfg_db_set_int(conf, "alarm", "alarm_h", alarm_h);
 
-   alarm_m = alarm_conf.default_min = 
+   alarm_m = alarm_conf.default_min =
      gtk_spin_button_get_value_as_int(alarm_conf.alarm_m);
    aud_cfg_db_set_int(conf, "alarm", "alarm_m", alarm_m);
 
@@ -206,13 +210,13 @@ void alarm_save(GtkButton *w, gpointer data)
        alarm_conf.day[daynum].flags = 0;
      else
        alarm_conf.day[daynum].flags = ALARM_OFF;
-   
+
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(alarm_conf.day[daynum].cb_def)))
        alarm_conf.day[daynum].flags |= ALARM_DEFAULT;
 
-     alarm_conf.day[daynum].hour = 
+     alarm_conf.day[daynum].hour =
        gtk_spin_button_get_value_as_int(alarm_conf.day[daynum].spin_hr);
-     alarm_conf.day[daynum].min = 
+     alarm_conf.day[daynum].min =
        gtk_spin_button_get_value_as_int(alarm_conf.day[daynum].spin_min);
 
      aud_cfg_db_set_int(conf, "alarm", day_flags[daynum], alarm_conf.day[daynum].flags);
@@ -277,8 +281,8 @@ void alarm_save(GtkButton *w, gpointer data)
    alarm_conf.reminder_msg = gtk_editable_get_chars(GTK_EDITABLE(alarm_conf.reminder),
        0, -1);
    aud_cfg_db_set_string(conf, "alarm", "reminder_msg", alarm_conf.reminder_msg);
-   
-   alarm_conf.reminder_on = 
+
+   alarm_conf.reminder_on =
      gtk_toggle_button_get_active(alarm_conf.reminder_cb);
    aud_cfg_db_set_bool(conf, "alarm", "reminder_on", alarm_conf.reminder_on);
 
@@ -493,7 +497,7 @@ static void alarm_configure(void)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alarm_conf.day[daynum].cb_def),
          alarm_conf.day[daynum].flags & ALARM_DEFAULT);
 
-   
+
      /* Changed to show default time instead of set time when ALARM_DEFAULT set,
       * as suggested by Mark Brown
       */
@@ -649,7 +653,7 @@ void alarm_current_volume(GtkButton *button, gpointer data)
 
    AUDDBG("on_current_button_clicked\n");
 
-   audacious_drct_get_volume_main(&vol);
+   aud_drct_get_volume_main(&vol);
 
    adj = gtk_range_get_adjustment(alarm_conf.volume);
    gtk_adjustment_set_value(adj, (gfloat)vol);
@@ -677,7 +681,7 @@ static inline pthread_t alarm_thread_create(void *(*start_routine)(void *), void
 
    if(detach != 0)
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-    
+
    pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED);
    pthread_attr_setschedpolicy(&attr, SCHED_OTHER);
    pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
@@ -698,7 +702,7 @@ static void *alarm_fade(void *arg)
    pthread_mutex_lock(&fader_lock);
 
    /* slide volume */
-   /* the Kaspar Giger way of fading, check the current mixer volume and 
+   /* the Kaspar Giger way of fading, check the current mixer volume and
     * increment from there so that if you have some other app lowering the
     * volume at the same time xmms-alarm will not ignore it.  If you have some
     * other app increasing the volume, then it could get louder that you expect
@@ -714,20 +718,20 @@ static void *alarm_fade(void *arg)
      inc = -1;
    else
      inc = 1;
-   
-   audacious_drct_set_volume_main((gint)vols->start);
+
+   aud_drct_set_volume_main((gint)vols->start);
    //for(i=0;i<(vols->end - vols->start);i++)
    for(i=0;i<adiff;i++)
    {
      //threadsleep((gfloat)fading / (vols->end - vols->start));
      threadsleep((gfloat)fading / (gfloat)adiff);
-     audacious_drct_get_volume_main(&v);
-     audacious_drct_set_volume_main(v + inc);
+     aud_drct_get_volume_main(&v);
+     aud_drct_set_volume_main(v + inc);
    }
    /* Setting the volume to the end volume sort of defeats the point if having
     * the code in there to allow other apps to control volume too :)
     */
-   //audacious_drct_set_volume_main((gint)vols->end);
+   //aud_drct_set_volume_main((gint)vols->end);
 
    /* and */
    pthread_mutex_unlock(&fader_lock);
@@ -755,7 +759,7 @@ static void *alarm_stop_thread( void *args )
    if (dialog_visible(alarm_dialog))
      gtk_widget_destroy(alarm_dialog);
 
-   audacious_drct_get_volume_main(&currvol),
+   aud_drct_get_volume_main(&currvol),
 
    /* fade back to zero */
    fade_vols.start = currvol;
@@ -765,13 +769,13 @@ static void *alarm_stop_thread( void *args )
    f_tid = alarm_thread_create(alarm_fade, &fade_vols, 0);
 
    pthread_join(f_tid, NULL);
-   audacious_drct_stop();
+   aud_drct_stop();
 
    /* might as well set the volume to something higher than zero so we
     * dont confuse the poor people who just woke up and cant work out why
     * theres no music playing when they press the little play button :)
     */
-   audacious_drct_set_volume_main(currvol);
+   aud_drct_set_volume_main(currvol);
 
    AUDDBG("alarm_stop done\n");
    return(NULL);
@@ -863,20 +867,20 @@ static void *alarm_start_thread(void *args)
        list.prev = list.next = NULL;
        list.data = playlist;
 
-       audacious_drct_pl_clear();
-       audacious_drct_pl_add(&list);
+       aud_drct_pl_clear();
+       aud_drct_pl_add_list (& list, -1);
      }
 
      if(fading)
      {
        fader fade_vols;
-       
+
        AUDDBG("Fading is true\n");
-       audacious_drct_set_volume_main(quietvol);
+       aud_drct_set_volume_main(quietvol);
 
        /* start playing */
        play_start = time(NULL);
-       audacious_drct_play();
+       aud_drct_play();
 
        /* fade volume */
        fade_vols.start = quietvol;
@@ -890,11 +894,11 @@ static void *alarm_start_thread(void *args)
        /* no fading */
 
        /* set volume */
-       audacious_drct_set_volume_main(volume);
+       aud_drct_set_volume_main(volume);
 
        /* start playing */
        play_start = time(NULL);
-       audacious_drct_play();
+       aud_drct_play();
      }
 
      if(alarm_conf.reminder_on == TRUE)
