@@ -28,21 +28,27 @@
 #define MAINWIN_DEFAULT_POS_X     100
 #define MAINWIN_DEFAULT_POS_Y     100
 
-typedef enum {
+enum {
     VIS_IN_TABS = 0,
     VIS_ON_LEFT,
     VIS_ON_RIGHT,
     VIS_ON_TOP,
     VIS_ON_BOTTOM
-} VisPosition;
+};
 
 typedef struct
 {
     gint player_x, player_y;
     gint player_width, player_height;
+    gint playlist_width, playlist_height;
     gboolean save_window_position;
     gboolean player_visible;
-    VisPosition vis_position;
+    gboolean playlist_visible;
+    gint vis_position;
+    gboolean multi_column_view;
+    gboolean infoarea_visible;
+    gboolean menu_visible;
+    gboolean statusbar_visible;
 } gtkui_cfg_t;
 
 extern gtkui_cfg_t config;
