@@ -1,5 +1,6 @@
 /*  Audacious - Cross-platform multimedia player
  *  Copyright (C) 2009 Tomasz Moń <desowin@gmail.com>
+ *  Copyright (C) 2010 Michał Lipski <tallica@o2.pl>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +18,16 @@
  *  Audacious or using our public API to be a derived work.
  */
 
-#include <audacious/plugin.h>
+#include "config.h"
+
+#include <audacious/audconfig.h>
+#include <audacious/debug.h>
+#include <audacious/playlist.h>
+#include <libaudcore/hook.h>
+
 #include "ui_playlist_model.h"
+#include "ui_playlist_widget.h"
+#include "playlist_util.h"
 
 static GObjectClass *parent_class = NULL;
 
@@ -111,12 +120,31 @@ ui_playlist_tree_model_init(GtkTreeModelIface *iface)
 static void
 ui_playlist_model_init(UiPlaylistModel *model)
 {
-    model->n_columns = PLAYLIST_N_COLUMNS;
+    if (multi_column_view)
+    {
+        model->n_columns = PLAYLIST_N_MULTI_COLUMNS;
 
-    model->column_types[PLAYLIST_COLUMN_NUM] = G_TYPE_UINT;
-    model->column_types[PLAYLIST_COLUMN_TEXT] = G_TYPE_STRING;
-    model->column_types[PLAYLIST_COLUMN_TIME] = G_TYPE_STRING;
-    model->column_types[PLAYLIST_COLUMN_WEIGHT] = PANGO_TYPE_WEIGHT;
+        model->column_types = g_new0(GType, PLAYLIST_N_MULTI_COLUMNS);
+        model->column_types[PLAYLIST_MULTI_COLUMN_NUM] = G_TYPE_UINT;
+        model->column_types[PLAYLIST_MULTI_COLUMN_ARTIST] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_MULTI_COLUMN_ALBUM] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_MULTI_COLUMN_TITLE] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_MULTI_COLUMN_TRACK_NUM] = G_TYPE_UINT;
+        model->column_types[PLAYLIST_MULTI_COLUMN_QUEUED] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_MULTI_COLUMN_TIME] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_MULTI_COLUMN_WEIGHT] = PANGO_TYPE_WEIGHT;
+    }
+    else
+    {
+        model->n_columns = PLAYLIST_N_COLUMNS;
+
+        model->column_types = g_new0(GType, PLAYLIST_N_COLUMNS);
+        model->column_types[PLAYLIST_COLUMN_NUM] = G_TYPE_UINT;
+        model->column_types[PLAYLIST_COLUMN_TEXT] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_COLUMN_QUEUED] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_COLUMN_TIME] = G_TYPE_STRING;
+        model->column_types[PLAYLIST_COLUMN_WEIGHT] = PANGO_TYPE_WEIGHT;
+    }
 
     model->num_rows = 0;
 
@@ -130,6 +158,9 @@ ui_playlist_model_finalize(GObject *object)
 
     ui_playlist_model_dissociate_hooks(model);
 
+    g_list_free (model->queue);
+    g_free(model->column_types);
+
     (* parent_class->finalize) (object);
 }
 
@@ -205,11 +236,36 @@ ui_playlist_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
     return path;
 }
 
+static void ui_playlist_model_get_value_time(UiPlaylistModel *model, GValue *value, gint position)
+{
+    gint length = aud_playlist_entry_get_length (model->playlist, position,
+     TRUE) / 1000;
+    gchar * len = g_strdup_printf("%02i:%02i", length / 60, length % 60);
+    g_value_set_string(value, len);
+    g_free(len);
+}
+
+static const gchar *ui_playlist_model_tuple_get_string(const Tuple *tuple, gint field)
+{
+    if (tuple == NULL)
+        return NULL;
+
+    return tuple_get_string(tuple, field, NULL);
+}
+
+static gint ui_playlist_model_tuple_get_int(const Tuple *tuple, gint field)
+{
+    if (tuple == NULL)
+        return 0;
+
+    return tuple_get_int(tuple, field, NULL);
+}
+
 static void
 ui_playlist_model_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter, gint column, GValue *value)
 {
     UiPlaylistModel *model;
-    gint n;
+    gint n, i;
 
     g_return_if_fail(UI_IS_PLAYLIST_MODEL(tree_model));
     g_return_if_fail(iter != NULL);
@@ -224,30 +280,93 @@ ui_playlist_model_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter, gint co
     if (n >= model->num_rows)
         g_return_if_reached();
 
-    switch (column)
+    if (multi_column_view)
     {
-      case PLAYLIST_COLUMN_NUM:
-        g_value_set_uint(value, n+1);
-        break;
-      case PLAYLIST_COLUMN_TEXT:
-        g_value_set_string(value, aud_playlist_entry_get_title(model->playlist, n));
-        break;
-      case PLAYLIST_COLUMN_TIME:
-      {
-        gint length = aud_playlist_entry_get_length(model->playlist, n) / 1000;
-        gchar * len = g_strdup_printf("%02i:%02i", length / 60, length % 60);
-        g_value_set_string(value, len);
-        g_free(len);
-        break;
-      }
-      case PLAYLIST_COLUMN_WEIGHT:
-        if (n == model->position)
-            g_value_set_enum(value, PANGO_WEIGHT_BOLD);
-        else
-            g_value_set_enum(value, PANGO_WEIGHT_NORMAL);
-        break;
-      default:
-        break;
+        const Tuple * tu = aud_playlist_entry_get_tuple (model->playlist, n,
+         TRUE);
+
+        switch (column)
+        {
+            case PLAYLIST_MULTI_COLUMN_NUM:
+                g_value_set_uint(value, n+1);
+                break;
+            case PLAYLIST_MULTI_COLUMN_ARTIST:
+                g_value_set_string(value, ui_playlist_model_tuple_get_string(tu, FIELD_ARTIST));
+                break;
+
+            case PLAYLIST_MULTI_COLUMN_ALBUM:
+                g_value_set_string(value, ui_playlist_model_tuple_get_string(tu, FIELD_ALBUM));
+                break;
+
+            case PLAYLIST_MULTI_COLUMN_TRACK_NUM:
+                g_value_set_uint(value, ui_playlist_model_tuple_get_int(tu, FIELD_TRACK_NUMBER));
+                break;
+
+            case PLAYLIST_MULTI_COLUMN_TITLE:
+            {
+                const gchar *title = ui_playlist_model_tuple_get_string(tu, FIELD_TITLE);
+
+                if (title == NULL)
+                    g_value_set_string (value, aud_playlist_entry_get_title
+                     (model->playlist, n, TRUE));
+                else
+                    g_value_set_string(value, title);
+                break;
+            }
+
+            case PLAYLIST_MULTI_COLUMN_QUEUED:
+                if ((i = aud_playlist_queue_find_entry (model->playlist, n)) < 0)
+                    g_value_set_string (value, "");
+                else
+                    g_value_take_string (value, g_strdup_printf ("#%d", 1 + i));
+                break;
+
+            case PLAYLIST_MULTI_COLUMN_TIME:
+                ui_playlist_model_get_value_time(model, value, n);
+                break;
+
+            case PLAYLIST_MULTI_COLUMN_WEIGHT:
+                if (n == model->position)
+                    g_value_set_enum(value, PANGO_WEIGHT_BOLD);
+                else
+                    g_value_set_enum(value, PANGO_WEIGHT_NORMAL);
+                break;
+            default:
+                break;
+        }
+    }
+    else
+    {
+        switch (column)
+        {
+            case PLAYLIST_COLUMN_NUM:
+                g_value_set_uint(value, n+1);
+                break;
+
+            case PLAYLIST_COLUMN_TEXT:
+                g_value_set_string (value, aud_playlist_entry_get_title
+                 (model->playlist, n, TRUE));
+                break;
+
+            case PLAYLIST_COLUMN_QUEUED:
+                if ((i = aud_playlist_queue_find_entry (model->playlist, n)) < 0)
+                    g_value_set_string (value, "");
+                else
+                    g_value_take_string (value, g_strdup_printf ("#%d", 1 + i));
+                break;
+
+            case PLAYLIST_COLUMN_TIME:
+                ui_playlist_model_get_value_time(model, value, n);
+                break;
+            case PLAYLIST_COLUMN_WEIGHT:
+                if (n == model->position)
+                    g_value_set_enum(value, PANGO_WEIGHT_BOLD);
+                else
+                    g_value_set_enum(value, PANGO_WEIGHT_NORMAL);
+                break;
+            default:
+                break;
+        }
     }
 }
 
@@ -358,15 +477,11 @@ ui_playlist_model_new(gint playlist)
 
     model->playlist = playlist;
     model->num_rows = aud_playlist_entry_count(playlist);
-
-    if (playlist == aud_playlist_get_active())
-    {
-        model->position = aud_playlist_get_position(playlist);
-    }
-    else
-    {
-        model->position = -1;
-    }
+    model->position = aud_playlist_get_position (playlist);
+    model->queue = NULL;
+    model->song_changed = FALSE;
+    model->focus_changed = FALSE;
+    model->selection_changed = FALSE;
 
     ui_playlist_model_associate_hooks(model);
 
@@ -432,28 +547,71 @@ ui_playlist_model_update_position(UiPlaylistModel *model, gint position)
     }
 }
 
+static void ui_playlist_model_playlist_rearraged(UiPlaylistModel *model)
+{
+    gtk_widget_queue_draw(GTK_WIDGET(playlist_get_treeview(model->playlist)));
+}
+
+static void ui_playlist_model_playlist_position (void * hook_data, void *
+ user_data)
+{
+    gint playlist = GPOINTER_TO_INT (hook_data);
+    UiPlaylistModel * model = user_data;
+
+    if (playlist == model->playlist)
+        model->song_changed = TRUE;
+}
+
+static void update_queue_row_changed (void * row, void * model)
+{
+    ui_playlist_model_row_changed (model, GPOINTER_TO_INT (row));
+}
+
+static void update_queue (UiPlaylistModel * model)
+{
+    gint i;
+
+    /* update previously queued rows */
+    g_list_foreach (model->queue, update_queue_row_changed, model);
+
+    g_list_free (model->queue);
+    model->queue = NULL;
+
+    for (i = aud_playlist_queue_count (model->playlist); i --; )
+        model->queue = g_list_prepend (model->queue, GINT_TO_POINTER
+         (aud_playlist_queue_get_entry (model->playlist, i)));
+
+    /* update currently queued rows */
+    g_list_foreach (model->queue, update_queue_row_changed, model);
+}
+
 static void
 ui_playlist_model_playlist_update(gpointer hook_data, gpointer user_data)
 {
     UiPlaylistModel *model = UI_PLAYLIST_MODEL(user_data);
+    GtkTreeView *treeview = playlist_get_treeview(model->playlist);
     gint type = GPOINTER_TO_INT(hook_data);
-    gint position;
 
     if (model->playlist != aud_playlist_get_active())
         return;
 
-    position = aud_playlist_get_position(model->playlist);
-    if (position != model->position)
-    {
-        ui_playlist_model_update_position(model, position);
-    }
+    ui_playlist_widget_block_updates ((GtkWidget *) treeview, TRUE);
+
+    gtk_tree_view_column_set_visible (g_object_get_data ((GObject *) treeview,
+     "number column"), aud_cfg->show_numbers_in_pl);
 
     if (type == PLAYLIST_UPDATE_STRUCTURE)
     {
         gint changed_rows;
         changed_rows = aud_playlist_entry_count(model->playlist) - model->num_rows;
 
-        if (changed_rows > 0)
+        AUDDBG("playlist structure update\n");
+
+        if (changed_rows == 0)
+        {
+            ui_playlist_model_playlist_rearraged(model);
+        }
+        else if (changed_rows > 0)
         {
             /* entries added */
             while (changed_rows != 0)
@@ -472,8 +630,37 @@ ui_playlist_model_playlist_update(gpointer hook_data, gpointer user_data)
             }
         }
 
-        model->num_rows = aud_playlist_entry_count(model->playlist);
+        ui_playlist_model_update_position (model, aud_playlist_get_position
+         (model->playlist));
     }
+    else if (type == PLAYLIST_UPDATE_METADATA)
+    {
+        AUDDBG("playlist metadata update\n");
+        ui_playlist_model_playlist_rearraged(model);
+    }
+    else if (type == PLAYLIST_UPDATE_SELECTION)
+        update_queue (model);
+
+    if (model->song_changed)
+    {
+        gint song = aud_playlist_get_position (model->playlist);
+
+        if (type != PLAYLIST_UPDATE_STRUCTURE) /* already did it? */
+            ui_playlist_model_update_position (model, song);
+
+        playlist_scroll_to_row (treeview, song);
+        model->song_changed = FALSE;
+    }
+
+    if (model->focus_changed)
+        treeview_set_focus_now (treeview, model->focus);
+    else if (model->selection_changed)
+        treeview_refresh_selection_now (treeview);
+
+    model->focus_changed = FALSE;
+    model->selection_changed = FALSE;
+
+    ui_playlist_widget_block_updates ((GtkWidget *) treeview, FALSE);
 }
 
 static void
@@ -499,14 +686,15 @@ ui_playlist_model_playlist_delete(gpointer hook_data, gpointer user_data)
 static void
 ui_playlist_model_associate_hooks(UiPlaylistModel *model)
 {
-    aud_hook_associate("playlist update", ui_playlist_model_playlist_update, model);
-    aud_hook_associate("playlist delete", ui_playlist_model_playlist_delete, model);
+    hook_associate("playlist update", ui_playlist_model_playlist_update, model);
+    hook_associate("playlist delete", ui_playlist_model_playlist_delete, model);
+    hook_associate("playlist position", ui_playlist_model_playlist_position, model);
 }
 
 static void
 ui_playlist_model_dissociate_hooks(UiPlaylistModel *model)
 {
-    aud_hook_dissociate_full("playlist update", ui_playlist_model_playlist_update, model);
-    aud_hook_dissociate_full("playlist delete", ui_playlist_model_playlist_delete, model);
+    hook_dissociate_full("playlist update", ui_playlist_model_playlist_update, model);
+    hook_dissociate_full("playlist delete", ui_playlist_model_playlist_delete, model);
+    hook_dissociate_full("playlist position", ui_playlist_model_playlist_position, model);
 }
-
